Package: adjustr
Encoding: UTF-8
Type: Package
Title: Stan Model Adjustments and Sensitivity Analyses using Importance
        Sampling
Version: 0.2.0
Authors@R: person("Cory", "McCartan", email = "mccartan@psu.edu",
  role = c("aut", "cre", "cph"))
Description: Assess the sensitivity of a Bayesian model (fitted using 'Stan'
    via 'rstan', 'brms', or 'cmdstanr') to the specification of its likelihood
    and priors. Users provide a series of alternate sampling specifications, and
    the package uses Pareto-smoothed importance sampling (PSIS) to estimate
    posterior quantities of interest under each specification, without needing
    to refit the model. Methods are based on Vehtari, Simpson, Gelman, Yao,
    and Gabry (2024) <doi:10.48550/arXiv.1507.02646>.
License: MIT + file LICENSE
Depends: R (>= 3.6.0), dplyr (>= 1.0.0)
Imports: rlang, tidyselect, rstan, loo
Suggests: ggplot2, extraDistr, tidyr, testthat, covr, knitr, rmarkdown
URL: https://corymccartan.com/adjustr/
BugReports: https://github.com/CoryMcCartan/adjustr/issues
LazyData: true
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-22 22:09:00 UTC; cmccartan
Author: Cory McCartan [aut, cre, cph]
Maintainer: Cory McCartan <mccartan@psu.edu>
Repository: CRAN
Date/Publication: 2026-05-29 09:20:02 UTC
Built: R 4.5.2; ; 2026-05-29 10:23:53 UTC; unix
