# Generated by roxygen2: do not edit by hand

S3method(arrange,adjustr_spec)
S3method(as.data.frame,adjustr_spec)
S3method(dplyr::filter,adjustr_spec)
S3method(length,adjustr_spec)
S3method(print,adjustr_spec)
S3method(pull,adjustr_weighted)
S3method(rename,adjustr_spec)
S3method(select,adjustr_spec)
S3method(slice,adjustr_spec)
S3method(stats::quantile,weighted.ecdf)
S3method(summarise,adjustr_weighted)
S3method(summarize,adjustr_weighted)
export(adjust_weights)
export(extract_samp_stmts)
export(get_resampling_idxs)
export(make_spec)
export(spec_plot)
import(dplyr)
import(rlang)
