% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_weights.R
\name{pull.adjustr_weighted}
\alias{pull.adjustr_weighted}
\title{Extract Weights From an \code{adjustr_weighted} Object}
\usage{
\method{pull}{adjustr_weighted}(.data, var = ".weights", name = NULL, ...)
}
\arguments{
\item{.data}{A table of data}

\item{var}{A variable, as in \code{\link[dplyr]{pull}}. The default returns
the \code{.weights} column, and if there is only one row, it returns the
first element of that column}

\item{name}{Ignored}

\item{...}{Ignored}
}
\value{
A numeric vector of weights (if a single row) or a list of numeric
  vectors.
}
\description{
This function modifies the default behavior of \code{dplyr::pull} to extract
the \code{.weights} column.
}
